/*  Copyright (C) MOXA Inc. All rights reserved.

    This software is distributed under the terms of the
    MOXA License.  See the file COPYING-MOXA for details.
*/
#include "os_dependent.h"
#include "list.h"

LIST*
list_new (void *p)
{
    LIST   *list = (LIST*) MALLOC (sizeof (LIST));

    if (list)
	{
		list->next = NULL;
		list->data = p;
	}
    return list;
}

LIST*
list_push (LIST * head, LIST * elem)
{
    elem->next = head;
    return elem;
}

LIST*
list_push_data (LIST * l, void *d)
{
    LIST   *list;

    list=list_new (d);
    return (list_push (l, list));
}

/* remove the element matching `data' from the list */
LIST*
list_delete (LIST * list, void *data)
{
    LIST  **ptr, *tmp;

    for (ptr = &list; *ptr; ptr = &(*ptr)->next)
    {
	if ((*ptr)->data == data)
	{
	    tmp = *ptr;
	    *ptr = (*ptr)->next;
	    FREE (tmp);
	    break;
	}
    }
    return list;
}

void
list_free (LIST * l, list_destroy_t cb)
{
    LIST   *t;

    while (l)
    {
	t = l;
	l = l->next;
	if (cb)
	    cb (t->data);
	FREE (t);
    }
}

#if 0 /* wipe out because they are not used */
LIST*
list_append (LIST * l, LIST * b)
{
    LIST  **r = &l;

    while (*r)
    {
	r = &(*r)->next;
    }
    *r = b;
    return l;
}

LIST*
list_append_data (LIST * l, void *d)
{
    LIST   *list;

    list=list_new (d);
    return (list_append (l, list));
}

int
list_count (LIST * list)
{
    int     count = 0;

    for (; list; list = list->next)
    {
	count++;
    }
    return count;
}

LIST*
list_find (LIST * list, void *data)
{
    for (; list; list = list->next)
    {
	if (list->data == data)
	    return list;
    }
    return 0;
}

void
list_foreach (LIST * list, list_callback_t func, void *arg)
{
    while (list)
    {
	func (list->data, arg);
	list = list->next;
    }
}
#endif
